#
# raccp.rb
#
#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2.
#

require 'racc/parser'


module Racc

  class RaccParser < Parser

    def initialize( racc )
      @yydebug     = racc.d_parse && Racc_debug_parser
      @ruletable   = racc.ruletable
      @symboltable = racc.symboltable

      @class_name = nil
      @super_class = 'Racc::Parser'
    end

    attr :class_name
    attr :super_class

    def parse( str )
      @scanner = RaccScanner.new( str )
      @scanner.debug = @yydebug

      do_parse
    end


    private

    def next_token
      @scanner.scan
    end

    def on_error( tok, val, _values )
      if val.respond_to? :id2name then
        v = val.id2name
      elsif String === val then
        v = val
      else
        v = val.inspect
      end
      raise ParseError, "#{@scanner.lineno}: unexpected token '#{v}'"
    end


##### racc 1.2.5 generates ###

racc_reduce_table = [
 0, 0, :racc_error,
 6, 23, :_reduce_1,
 1, 24, :_reduce_2,
 3, 24, :_reduce_3,
 1, 27, :_reduce_4,
 4, 27, :_reduce_5,
 0, 25, :_reduce_none,
 2, 25, :_reduce_none,
 3, 28, :_reduce_8,
 1, 28, :_reduce_none,
 2, 28, :_reduce_10,
 2, 28, :_reduce_11,
 2, 28, :_reduce_12,
 2, 29, :_reduce_13,
 3, 29, :_reduce_14,
 3, 30, :_reduce_15,
 3, 30, :_reduce_16,
 1, 34, :_reduce_none,
 2, 34, :_reduce_none,
 2, 35, :_reduce_19,
 2, 35, :_reduce_20,
 2, 35, :_reduce_21,
 1, 32, :_reduce_22,
 2, 32, :_reduce_23,
 2, 32, :_reduce_none,
 1, 31, :_reduce_25,
 1, 31, :_reduce_26,
 1, 26, :_reduce_27,
 0, 26, :_reduce_none,
 1, 36, :_reduce_29,
 2, 36, :_reduce_30,
 2, 36, :_reduce_31,
 2, 36, :_reduce_32,
 1, 37, :_reduce_none,
 1, 37, :_reduce_34,
 2, 37, :_reduce_35,
 1, 37, :_reduce_36,
 1, 33, :_reduce_37,
 2, 33, :_reduce_38 ]

racc_reduce_n = 39

racc_shift_n = 63

racc_action_table = [
    24,    54,    24,    24,    24,    24,    22,    40,    22,    22,
    22,    22,    48,    49,    50,    51,    44,    44,    24,    24,
    41,    47,    24,    24,    22,    22,    43,    15,    22,    22,
    44,    44,    11,    12,    13,    14,    24,    17,    18,    24,
    24,    24,    22,    24,    46,    22,    22,    22,    60,    22,
    38,    34,    35,    58,    38,    34,    35,    38,    34,    35,
     8,     9,     9,    38,    34,    35,    30,    21,     5,    61,
    10,     6,     5,     1 ]

racc_action_check = [
    32,    32,    15,    11,    55,    56,    32,    21,    15,    11,
    55,    56,    32,    32,    32,    32,    55,    56,    59,    27,
    23,    31,    23,    12,    59,    27,    25,     7,    23,    12,
    59,    27,     7,     7,     7,     7,    50,     7,     7,    34,
    38,    35,    50,    13,    29,    34,    38,    35,    39,    13,
    39,    39,    39,    36,    36,    36,    36,    17,    17,    17,
     4,    20,     4,    18,    18,    18,    14,     9,     8,    42,
     6,     2,     1,     0 ]

racc_action_pointer = [
    71,    66,    71,   nil,    55,   nil,    70,    24,    62,    60,
   nil,    -3,    17,    37,    60,    -4,   nil,    42,    48,   nil,
    54,     1,   nil,    16,   nil,    14,   nil,    13,   nil,    38,
   nil,    17,    -6,   nil,    33,    35,    39,   nil,    34,    35,
   nil,   nil,    57,   nil,   nil,   nil,   nil,   nil,   nil,   nil,
    30,   nil,   nil,   nil,   nil,    -2,    -1,   nil,   nil,    12,
   nil,   nil,   nil ]

racc_action_default = [
   -39,   -39,   -39,    -6,    -2,    -4,   -39,   -39,   -39,   -39,
    63,   -39,   -39,   -39,   -39,   -28,    -7,   -39,   -39,    -9,
    -3,   -39,   -26,   -39,   -25,   -39,   -10,   -11,   -22,   -12,
   -37,   -39,   -27,   -29,   -39,   -39,   -39,   -17,   -39,   -39,
    -5,    -8,   -39,   -13,   -24,   -23,   -38,    -1,   -34,   -31,
   -39,   -36,   -30,   -33,   -32,   -20,   -21,   -18,   -15,   -19,
   -16,   -14,   -35 ]

racc_goto_table = [
    25,    26,    27,     4,    33,    57,    36,    39,    57,    19,
    20,    23,    42,    16,     2,    31,    45,    29,     7,     3,
    32,    53,    52,    55,    56,   nil,   nil,    59,   nil,   nil,
   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,    62,
   nil,   nil,   nil,   nil,    45,    45,   nil,   nil,    45 ]

racc_goto_check = [
     9,     9,    10,     5,     9,    13,    12,    12,    13,     8,
     5,     7,     9,     6,     1,     4,     9,    11,     3,     2,
    14,     9,    15,    10,    10,   nil,   nil,    10,   nil,   nil,
   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,     9,
   nil,   nil,   nil,   nil,     9,     9,   nil,   nil,     9 ]

racc_goto_pointer = [
   nil,    14,    18,    15,     0,     2,     6,     0,     2,   -11,
   -11,     3,   -11,   -31,     5,   -10 ]

racc_goto_default = [
   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,    28,
   nil,   nil,   nil,    37,   nil,   nil ]

racc_token_table = {
 false => 0,
 Object.new => 1,
 :XCLASS => 2,
 :XRULE => 3,
 :XEND => 4,
 "<" => 5,
 :XSYMBOL => 6,
 ":" => 7,
 :XCONV => 8,
 :XSTART => 9,
 :XTOKEN => 10,
 :XOPTION => 11,
 :STRING => 12,
 :XPRECHIGH => 13,
 :XPRECLOW => 14,
 :XLEFT => 15,
 :XRIGHT => 16,
 :XNONASSOC => 17,
 "|" => 18,
 ";" => 19,
 "=" => 20,
 :ACTION => 21 }

racc_use_result_var = true

racc_nt_base = 22

Racc_arg = [
 racc_action_table,
 racc_action_check,
 racc_action_default,
 racc_action_pointer,
 racc_goto_table,
 racc_goto_check,
 racc_goto_default,
 racc_goto_pointer,
 racc_nt_base,
 racc_reduce_table,
 racc_token_table,
 racc_shift_n,
 racc_reduce_n,
 racc_use_result_var ]

Racc_debug_parser = false

##### racc system variables end #####

 # reduce 0 omitted

module_eval <<'.,.,', '(build)', 83
  def _reduce_1( val, _values, result )
      @ruletable.end_register_rule
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 87
  def _reduce_2( val, _values, result )
      @class_name = val[0]
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 90
  def _reduce_3( val, _values, result )
      @class_name = val[0]
      @super_class = val[2]
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 95
  def _reduce_4( val, _values, result )
      result = result.id2name
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 98
  def _reduce_5( val, _values, result )
      result << '::' << val[3].id2name
   result
  end
.,.,

 # reduce 6 omitted

 # reduce 7 omitted

module_eval <<'.,.,', '(build)', 108
  def _reduce_8( val, _values, result )
      @symboltable.end_register_conv
   result
  end
.,.,

 # reduce 9 omitted

module_eval <<'.,.,', '(build)', 114
  def _reduce_10( val, _values, result )
      @ruletable.register_start val[1]
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 118
  def _reduce_11( val, _values, result )
      @symboltable.register_token val[1]
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 122
  def _reduce_12( val, _values, result )
      val[1].each do |s|
        @ruletable.register_option s.to_s
      end
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 129
  def _reduce_13( val, _values, result )
      @symboltable.register_conv( val[0], val[1] )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 133
  def _reduce_14( val, _values, result )
      @symboltable.register_conv( val[1], val[2] )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 138
  def _reduce_15( val, _values, result )
      @symboltable.end_register_prec( true )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 142
  def _reduce_16( val, _values, result )
      @symboltable.end_register_prec( false )
   result
  end
.,.,

 # reduce 17 omitted

 # reduce 18 omitted

module_eval <<'.,.,', '(build)', 152
  def _reduce_19( val, _values, result )
      @symboltable.register_prec( :Left, val[1] )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 156
  def _reduce_20( val, _values, result )
      @symboltable.register_prec( :Right, val[1] )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 160
  def _reduce_21( val, _values, result )
      @symboltable.register_prec( :Nonassoc, val[1] )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 165
  def _reduce_22( val, _values, result )
      result = val
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 169
  def _reduce_23( val, _values, result )
      result.push val[1]
   result
  end
.,.,

 # reduce 24 omitted

module_eval <<'.,.,', '(build)', 175
  def _reduce_25( val, _values, result )
      result = @symboltable.get( result )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 179
  def _reduce_26( val, _values, result )
      result = @symboltable.get( eval '"' + val[0] + '"' )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 184
  def _reduce_27( val, _values, result )
      unless result.empty? then
        @ruletable.register_rule_from_array result
      end
   result
  end
.,.,

 # reduce 28 omitted

module_eval <<'.,.,', '(build)', 193
  def _reduce_29( val, _values, result )
      result = val
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 197
  def _reduce_30( val, _values, result )
      result.push val[1]
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 201
  def _reduce_31( val, _values, result )
      unless result.empty? then
        @ruletable.register_rule_from_array result
      end
      result.clear
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 208
  def _reduce_32( val, _values, result )
      pre = result.pop
      unless result.empty? then
        @ruletable.register_rule_from_array result
      end
      result = [pre]
   result
  end
.,.,

 # reduce 33 omitted

module_eval <<'.,.,', '(build)', 219
  def _reduce_34( val, _values, result )
      result = OrMark.new( @scanner.lineno )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 223
  def _reduce_35( val, _values, result )
      result = Prec.new( val[1], @scanner.lineno )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 227
  def _reduce_36( val, _values, result )
      result = UserAction.new( *result )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 231
  def _reduce_37( val, _values, result )
      result = [ result.id2name ]
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 234
  def _reduce_38( val, _values, result )
      result.push val[1].id2name
   result
  end
.,.,

 def _reduce_none( val, _values, result )
  result
 end

  end   # class RaccParser

end   # module Racc
